﻿using Oxide.Core;
using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Oxide.Plugins
{
    // Edit by Mroczny
	// Only for Hurtworld Legacy!
    [Info("Admin Tools", "Noviets", "1.4.2")]
    [Description("Kick, ban, temp ban, freeze and godmode with permissions")]
    public class AdminTools : HurtworldPlugin
    {
        private new void LoadDefaultMessages()
        {
            // English
            lang.RegisterMessages(new Dictionary<string, string>
            {
                ["nopermission"] = "You dont have Permission to do this!",
                ["playernotfound"] = "That player does not exist, or is not online.",
                ["noreason"] = "You must provide a reason to kick.",
                ["banusage"] = "Usage: /ban <Player|IP|SteamID>",
                ["unbanusage"] = "Usage: /unban <SteamID>",
                ["kickusage"] = "Usage: /kick <Player|SteamID>",
                ["kicked"] = "<color=#3478bc>{player}</color> has been kicked for: <color=#ffa500>{Reason}</color>",
                ["banned"] = "<color=#3478bc>{player}</color> has been banned.",
                ["unbanned"] = "You have Unbanned SteamID: {ID}",
                ["banmsg"] = "You have been Banned.",
                ["noclipenabled"] = "Enabled NoClip",
                ["noclipdisabled"] = "Disabled NoClip",
                ["healself"] = "Healed",
                ["healother"] = "Healed: {player}",
                ["healusage"] = "Usage: /heal <Player>",
                ["targetisadmin"] = "Player is admin!",
                ["godenabled"] = "Enabled Godmode",
                ["goddisabled"] = "Disabled Godmode",
                ["godkill"] = "Killing in Godmode.",
                ["frozenusage"] = "Usage: /freeze <Player>",
                ["frozenmsg"] = "You have Frozen {player}.",
                ["unfrozenmsg"] = "{player} is no longer Frozen.",
                ["frozen"] = "<color=#ff0000>You have been Frozen.</color>",
                ["unfrozen"] = "<color=#ff0000>You are no longer Frozen.</color>",
                ["notvalidnumber"] = "{arg} is not a valid number. ",
                ["infamyerror"] = "Usage: /infamy <Player|all> <amount>",
                ["infamyset"] = " {player} infamy set to {infamy}",
                ["infamysetall"] = "All players infamy set to {infamy}"
            }, this);

            // Polish
            lang.RegisterMessages(new Dictionary<string, string>
            {
                ["nopermission"] = "Nie posiadasz uprawnień do tego!",
                ["playernotfound"] = "<color=#B2B2B2>Gracz jest offline.</color>",
                ["noreason"] = "<color=#B2B2B2>Musisz podać powód, aby wyrzucić gracza.</color>",
                ["banusage"] = "<color=#B2B2B2>Poprawne użycie:</color> /ban <gracz|IP|SteamID>",
                ["unbanusage"] = "<color=#B2B2B2>Poprawne użycie:</color> /unban <SteamID>",
                ["kickusage"] = "<color=#B2B2B2>Poprawne użycie:</color> /kick <gracz|SteamID>",
                ["kicked"] = "<color=#3478bc>{player}</color><color=#B2B2B2> został wyrzucony za </color>{Reason}",
                ["banned"] = "<color=#3478bc>{player}</color><color=#B2B2B2> został zbanowany</color>.",
                ["unbanned"] = "<color=#B2B2B2>Odbanowałeś SteamID:</color> {ID}",
                ["banmsg"] = "\nZostałeś zbanowany.\n",
                ["noclipenabled"] = "Włączono NoClip",
                ["noclipdisabled"] = "Wyłączono NoClip",
                ["healself"] = "<color=#5B5B5B>»</color> <color=#B2B2B2>Uleczono</color>",
                ["healother"] = "<color=#5B5B5B>»</color> <color=#B2B2B2>Uleczyłeś</color> {player}",
                ["healusage"] = "<color=#B2B2B2>Poprawne użycie:</color> /heal <gracz>",
                ["targetisadmin"] = "Gracz jest adminem!",
                ["godenabled"] = "Włączono Godmode",
                ["goddisabled"] = "Wyłączono Godmode",
                ["godkill"] = "Zabijanie z użyciem nieśmiertelności.",
                ["frozenusage"] = "<color=#B2B2B2>Poprawne użycie:</color> /freeze <gracz>",
                ["frozenmsg"] = "<color=#5B5B5B>»</color> Zamroziłeś {player}.",
                ["unfrozenmsg"] = "<color=#5B5B5B>»</color> {player} nie jest już zamrożony.",
                ["frozen"] = "<color=#AF2424>Zostałeś zamrożony.</color>",
                ["unfrozen"] = "<color=#AF2424>Zostałeś odmrożony.</color>",
                ["notvalidnumber"] = "{arg} to nie jest prawidłowy numer.",
                ["infamyerror"] = "<color=#B2B2B2>Poprawne użycie:</color> /infamy <nick|all> <ilość>",
                ["infamyset"] = "<color=#5B5B5B>»</color> Niesławność gracza: {player} ustawiono na: {infamy}",
                ["infamysetall"] = "<color=#5B5B5B>»</color> Niesławność wszystkim graczom ustawiono na: {infamy}"
            }, this, "pl");
        }

        protected override void LoadDefaultConfig()
        {
            Config["ShowConsoleMsg"] = false;
            Config["KillingInGodBan"] = false;
            Config["KillingInGodKick"] = false;
            Config["LogKicks"] = false;
            Config["LogBans"] = false;
            Config["LogUnBan"] = false;
            Config["LogGodMode"] = false;
            Config["LogNoClip"] = false;
            Config["LogHeal"] = false;
            Config["LogInfamy"] = false;
            Config["LogFreeze"] = false;
            Config["LogKillingInGodMode"] = false;

            SaveConfig();
        }

        private string GetNameOfObject(UnityEngine.GameObject obj)
        {
            GameManager ManagerInstance = GameManager.Instance;
            return ManagerInstance.GetDescriptionKey(obj);
        }

        public static List<ulong> Godlist = new List<ulong>();
        private string Msg(string msg, object SteamId = null) => lang.GetMessage(msg, this, SteamId?.ToString());

        private void Init()
        {
            globallog = Interface.Oxide.DataFileSystem.ReadObject<List<string>>("AdminTools/GlobalLog");

            permission.RegisterPermission("admintools.kick", this);
            permission.RegisterPermission("admintools.ban", this);
            permission.RegisterPermission("admintools.godmode", this);
            permission.RegisterPermission("admintools.freeze", this);
            permission.RegisterPermission("admintools.infamy", this);
            permission.RegisterPermission("admintools.noclip", this);
        }

        private List<string> globallog = new List<string>();
        private void SaveGlobalLog() => Interface.Oxide.DataFileSystem.WriteObject("AdminTools/GlobalLog", globallog);
        private void SaveGodlist() => Interface.Oxide.DataFileSystem.WriteObject("AdminTools/Godlist", Godlist);

        private void OnPlayerDisconnected(PlayerSession session)
        {
            if (Godlist.Contains((ulong)session.SteamId))
            {
                Godlist.Remove((ulong)session.SteamId);
                SaveGodlist();
            }
        }

        private PlayerSession GetSession(String source)
        {
            Match IPCheck = Regex.Match(source, @"\b(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\b");

            foreach (PlayerSession session in GameManager.Instance.GetSessions().Values)
            {
                if (session != null && session.IsLoaded)
                {
                    if (source.ToLower() == session.Identity.Name.ToLower())
                    {
                        return session;
                    }
                }
            }

            foreach (PlayerSession session in GameManager.Instance.GetSessions().Values)
            {
                if (session != null && session.IsLoaded)
                {
                    if (IPCheck.Success)
                    {
                        if (source == session.Player.ipAddress)
                        {
                            return session;
                        }
                    }
                    else if (source == session.SteamId.ToString())
                    {
                        return session;
                    }
                    else if (session.Identity.Name.ToLower().Contains(source.ToLower()))
                    {
                        return session;
                    }
                }
            }
            return null;
        }

        private void OnPlayerDeath(PlayerSession session, EntityEffectSourceData source)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.godmode") || session.IsAdmin)
            {
                string KillName = GetNameOfObject(source.EntitySource);
                if (KillName != "")
                {
                    string Killer = KillName.Replace("(P)", "");
                    if ((bool)Config["KillingInGodBan"] || (bool)Config["KillingInGodKick"])
                    {
                        PlayerSession person = GetSession(Killer);
                        if (person != null)
                        {
                            ulong ID = (ulong)person.SteamId;
                            if (Godlist.Contains(ID))
                            {
                                if ((bool)Config["LogKillingInGodMode"])
                                {
                                    globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") killed " + person.Identity.Name + " (" + person.SteamId + ") while in GodMode");
                                    SaveGlobalLog();
                                }
                                if ((bool)Config["KillingInGodKick"])
                                {
                                    GameManager.Instance.KickPlayer(ID.ToString(), Msg("godkill"));
                                }

                                if (!person.IsAdmin)
                                {
                                    if ((bool)Config["KillingInGodBan"])
                                    {
                                        ConsoleManager.Instance.ExecuteCommand("ban " + ID);
                                        GameManager.Instance.KickPlayer(ID.ToString(), Msg("godkill"));
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private void OnPlayerTakeDamage(PlayerSession session, EntityEffectSourceData source)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.godmode") || session.IsAdmin)
            {
                if (Godlist.Contains((ulong)session.SteamId))
                {
                    source.Value = 0;
                }
            }
        }

        [ChatCommand("kick")]
        private void KickCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.kick") || session.IsAdmin)
            {
                if (args.Length == 0)
                {
                    hurt.SendChatMessage(session, null, Msg("kickusage", session.SteamId));
                    return;
                }

                if (args.Length >= 2)
                {
                    PlayerSession person = GetSession(args[0]);
                    if (person != null)
                    {
                        if (!person.IsAdmin)
                        {
                            string ID = person.SteamId.ToString();
                            string reason = string.Join(" ", args, 1, args.Length - 1);

                            hurt.BroadcastChat(null, Msg("kicked", session.SteamId).Replace("{player}", person.Identity.Name).Replace("{Reason}", reason));
                            GameManager.Instance.KickPlayer(ID, reason);
                            if ((bool)Config["ShowConsoleMsg"])
                            {
                                Puts(session.Identity.Name + " kicked " + person.Identity.Name);
                            }

                            if ((bool)Config["LogKicks"])
                            {
                                globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") KICKED " + person.Identity.Name + " (" + person.SteamId + ") for " + reason);
                                SaveGlobalLog();
                            }
                        }
                        else
                        {
                            hurt.SendChatMessage(session, null, Msg("targetisadmin", session.SteamId));
                        }
                    }
                    else
                    {
                        hurt.SendChatMessage(session, null, Msg("playernotfound", session.SteamId));
                    }
                }
                else
                {
                    hurt.SendChatMessage(session, null, Msg("noreason", session.SteamId));
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }

        [ChatCommand("ban")]
        private void BanCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.ban") || session.IsAdmin)
            {
                if (args.Length == 0)
                {
                    hurt.SendChatMessage(session, null, Msg("banusage", session.SteamId));
                    return;
                }
                if (args.Length == 1)
                {
                    PlayerSession person = GetSession(args[0]);
                    if (person != null)
                    {
                        if (!person.IsAdmin)
                        {
                            string ID = args[0];
                            hurt.BroadcastChat(null, Msg("banned").Replace("{player}", "SteamID: " + args[0]));
                            ConsoleManager.Instance.ExecuteCommand("ban " + ID);
                            GameManager.Instance.KickPlayer(ID, Msg("banmsg", session.SteamId));
                            if ((bool)Config["ShowConsoleMsg"])
                            {
                                Puts(session.Identity.Name + " banned SteamID: " + ID);
                            }

                            if ((bool)Config["LogBans"])
                            {
                                globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") BANNED (" + ID + ")");
                                SaveGlobalLog();
                            }
                        }
                        else
                        {
                            hurt.SendChatMessage(session, null, Msg("targetisadmin", session.SteamId));
                        }
                    }
                    else
                    {
                        hurt.SendChatMessage(session, null, Msg("playernotfound", session.SteamId));
                    }
                }
                else
                {
                    hurt.SendChatMessage(session, null, Msg("banusage", session.SteamId));
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }


        [ChatCommand("unban")]
        private void UnbanCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.ban") || session.IsAdmin)
            {
                if (args.Length == 1)
                {
                    string ID = args[0];
                    ConsoleManager.Instance.ExecuteCommand("unban " + ID);
                    hurt.SendChatMessage(session, null, Msg("unbanned", session.SteamId).Replace("{ID}", ID));

                    if ((bool)Config["ShowConsoleMsg"])
                    {
                        Puts(session.Identity.Name + " unbanned: " + ID);
                    }

                    if ((bool)Config["LogUnBan"])
                    {
                        globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") UNBANNED " + ID);
                        SaveGlobalLog();
                    }
                }
                else
                {
                    hurt.SendChatMessage(session, null, Msg("unbanusage", session.SteamId));
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }

        [ChatCommand("freeze")]
        private void FreezeCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.freeze") || session.IsAdmin)
            {
                if (args.Length == 0)
                {
                    hurt.SendChatMessage(session, null, Msg("frozenusage", session.SteamId));
                    return;
                }

                if (args.Length == 1)
                {
                    PlayerSession target = GetSession(args[0]);
                    if (target != null)
                    {
                        CharacterMotorSimple motor = target.WorldPlayerEntity.GetComponent<CharacterMotorSimple>();

                        if (motor.CanMove)
                        {
                            motor.CanMove = false;
                            hurt.SendChatMessage(session, null, Msg("frozenmsg", session.SteamId).Replace("{player}", target.Identity.Name));
                            hurt.SendChatMessage(target, null, Msg("frozen", target.SteamId));

                            if ((bool)Config["LogFreeze"])
                            {
                                globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") FROZE " + target.Identity.Name + " (" + target.SteamId + ")");
                                SaveGlobalLog();
                            }
                        }
                        else
                        {
                            motor.CanMove = true;
                            hurt.SendChatMessage(session, null, Msg("unfrozenmsg", session.SteamId).Replace("{player}", target.Identity.Name));
                            hurt.SendChatMessage(target, null, Msg("unfrozen", target.SteamId));
                            if ((bool)Config["LogFreeze"])
                            {
                                globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") UNFROZE " + target.Identity.Name + " (" + target.SteamId + ")");
                                SaveGlobalLog();
                            }
                        }
                    }
                    else
                    {
                        hurt.SendChatMessage(session, null, Msg("playernotfound", session.SteamId));
                    }
                }
                else
                {
                    hurt.SendChatMessage(session, null, Msg("frozenusage", session.SteamId));
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }

        [ChatCommand("godmode")]
        private void GodmodeCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.godmode") || session.IsAdmin)
            {
                if (!Godlist.Contains((ulong)session.SteamId))
                {
                    Heal(session);
                    AlertManager.Instance.GenericTextNotificationServer(Msg("godenabled", session.SteamId), session.Player);
                    Godlist.Add((ulong)session.SteamId);
                    SaveGodlist();

                    if ((bool)Config["ShowConsoleMsg"])
                    {
                        Puts(session.Identity.Name + " turned GODMODE on");
                    }

                    if ((bool)Config["LogGodMode"])
                    {
                        globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") turned GODMODE on");
                        SaveGlobalLog();
                    }
                }
                else
                {
                    Heal(session);
                    AlertManager.Instance.GenericTextNotificationServer(Msg("goddisabled", session.SteamId), session.Player);
                    Godlist.Remove((ulong)session.SteamId);
                    SaveGodlist();

                    if ((bool)Config["ShowConsoleMsg"])
                    {
                        Puts(session.Identity.Name + " turned GODMODE off");
                    }

                    if ((bool)Config["LogGodMode"])
                    {
                        globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") turned GODMODE off");
                    }
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }

        [ChatCommand("noclip")]
        private void NoClipCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.noclip") || session.IsAdmin)
            {
                if (session.WorldPlayerEntity.gameObject.layer != 12)
                {
                    session.WorldPlayerEntity.gameObject.layer = 12;
                    AlertManager.Instance.GenericTextNotificationServer(Msg("noclipenabled", session.SteamId), session.Player);

                    if ((bool)Config["LogNoClip"])
                    {
                        globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") turned NOCLIP on");
                        SaveGlobalLog();
                    }
                }
                else
                {
                    session.WorldPlayerEntity.gameObject.layer = 17;
                    AlertManager.Instance.GenericTextNotificationServer(Msg("noclipdisabled", session.SteamId), session.Player);

                    if ((bool)Config["LogNoClip"])
                    {
                        globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") turned NOCLIP off");
                        SaveGlobalLog();
                    }
                }
            }
        }

        private void Heal(PlayerSession player)
        {
            EntityStats stats = player.WorldPlayerEntity.GetComponent<EntityStats>();

            stats.GetFluidEffect(EEntityFluidEffectType.ColdBar).SetValue(0f);
            stats.GetFluidEffect(EEntityFluidEffectType.Radiation).SetValue(0f);
            stats.GetFluidEffect(EEntityFluidEffectType.HeatBar).SetValue(0f);
            stats.GetFluidEffect(EEntityFluidEffectType.Dampness).SetValue(0f);
            stats.GetFluidEffect(EEntityFluidEffectType.Hungerbar).SetValue(0f);
            stats.GetFluidEffect(EEntityFluidEffectType.Nutrition).SetValue(100f);
            stats.GetFluidEffect(EEntityFluidEffectType.InternalTemperature).Reset(true);
            stats.GetFluidEffect(EEntityFluidEffectType.ExternalTemperature).Reset(true);
            stats.GetFluidEffect(EEntityFluidEffectType.Toxin).SetValue(0f);
            stats.GetFluidEffect(EEntityFluidEffectType.Health).SetValue(100f);
            stats.RemoveBinaryEffect(EEntityBinaryEffectType.BrokenLeg);
            stats.RemoveBinaryEffect(EEntityBinaryEffectType.Wet);
        }

        private void HealAll()
        {
            foreach (PlayerSession session in GameManager.Instance.GetSessions().Values)
            {
                if (session != null && session.IsLoaded)
                {
                    Heal(session);
                }
            }
        }

        [ChatCommand("heal")]
        private void HealCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.godmode") || session.IsAdmin)
            {
                if (args.Length > 1)
                {
                    hurt.SendChatMessage(session, null, Msg("healusage", session.SteamId));
                    return;
                }
                if (args.Length == 0)
                {
                    Heal(session);
                    hurt.SendChatMessage(session, null, Msg("healself", session.SteamId));
                }
                if (args.Length == 1)
                {
                    PlayerSession target = GetSession(args[0]);
                    if (target != null)
                    {
                        Heal(target);
                        hurt.SendChatMessage(session, null, Msg("healother", session.SteamId).Replace("{player}", target.Identity.Name));
                        if ((bool)Config["LogHeal"])
                        {
                            globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") HEALED " + target.Identity.Name + " (" + target.SteamId + ")");
                            SaveGlobalLog();
                        }
                    }
                    else
                    {
                        hurt.SendChatMessage(session, null, Msg("playernotfound", session.SteamId));
                    }
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }

        [ChatCommand("infamy")]
        private void InfamyCommand(PlayerSession session, string command, string[] args)
        {
            if (permission.UserHasPermission(session.SteamId.ToString(), "admintools.infamy") || session.IsAdmin)
            {
                if (args.Length == 2)
                {
                    float infamy = 0f;
                    try { infamy = Convert.ToSingle(args[1]); } catch { hurt.SendChatMessage(session, null, Msg("notvalidnumber", session.SteamId).Replace("{arg}", args[1])); return; }
                    if (args[0].ToLower() == "all")
                    {
                        foreach (PlayerSession ses in GameManager.Instance.GetSessions().Values)
                        {
                            if (ses.IsLoaded)
                            {
                                EntityStats stats = ses.WorldPlayerEntity.GetComponent<EntityStats>();
                                stats.GetFluidEffect(EEntityFluidEffectType.Infamy).SetValue(infamy);
                            }
                        }
                        hurt.SendChatMessage(session, null, Msg("infamysetall", session.SteamId).Replace("{infamy}", infamy.ToString()));

                        if ((bool)Config["LogInfamy"])
                        {
                            globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") set Infamy on ALL PLAYERS to " + infamy);
                            SaveGlobalLog();
                        }
                    }
                    else
                    {
                        PlayerSession target = GetSession(args[0]);
                        EntityStats stats = target.WorldPlayerEntity.GetComponent<EntityStats>();
                        stats.GetFluidEffect(EEntityFluidEffectType.Infamy).SetValue(infamy);
                        hurt.SendChatMessage(session, null, Msg("infamyset", session.SteamId).Replace("{player}", target.Identity.Name).Replace("{infamy}", infamy.ToString()));
                        if ((bool)Config["LogInfamy"])
                        {
                            globallog.Add("[" + DateTime.Now + "] " + session.Identity.Name + " (" + session.SteamId + ") set Infamy on " + target.Identity.Name + " (" + target.SteamId + ") to " + infamy);
                            SaveGlobalLog();
                        }
                    }
                }
                else
                {
                    hurt.SendChatMessage(session, null, Msg("infamyerror", session.SteamId));
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }
    }
}
